#version 120

/*- Modified Chocapic13 Shader Lite By Airloocke42 -*/

varying vec4 texcoord;
varying vec3 sunlight;
varying vec3 ambient_color;

uniform sampler2D depthtex0;
uniform sampler2D gaux1;
uniform sampler2D gaux2;
uniform sampler2D gaux4;
uniform sampler2D gnormal;
uniform sampler2D gdepth;
uniform sampler2D gcolor;

uniform sampler2D composite;

uniform vec3 cameraPosition;
uniform vec3 previousCameraPosition;
uniform vec3 sunPosition;
uniform vec3 moonPosition;

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferPreviousProjection;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferPreviousModelView;

uniform ivec2 eyeBrightness;

uniform int isEyeInWater;
uniform int worldTime;

uniform float aspectRatio;
uniform float near;
uniform float far;
uniform float viewWidth;
uniform float viewHeight;
uniform float rainStrength;
uniform float wetness;
uniform float frameTimeCounter;

vec3 sunPos = sunPosition;

uniform int fogMode;

float pw = 1.0/ viewWidth;
float ph = 1.0/ viewHeight;

float rainx = clamp(rainStrength, 0.0f, 1.0f)/1.0f;
float wetx  = clamp(wetness, 0.0f, 1.0f);

float sky_lightmap = texture2D(gaux1,texcoord.xy).r;

vec3 fogclr = pow(mix(vec3(0.5,0.5,1.0),vec3(0.3,0.3,0.3),rainStrength)*ambient_color,vec3(2.2));

vec3 fragpos = vec3(texcoord.st, texture2D(depthtex0, texcoord.st).r);
vec3 normal = texture2D(gnormal, texcoord.st).rgb * 2.0 - 1.0;

float matflag = texture2D(gaux1,texcoord.xy).g;

float ld(float depth) { return (2.0 * near) / (far + near - depth * (far - near)); }

float A = 0.15, B = 0.25, C = 0.10, D = 0.20, E = 0.02, F = 0.30, W = 16.0;

vec3 Uncharted2Tonemap(vec3 x) { return ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F; }

vec3 nvec3(vec4 pos) { return pos.xyz/pos.w; }

	float sunre = texture2D(composite,texcoord.xy).a*16.0;

	float time = float(worldTime);
	float transition_fading = 1.0-(clamp((time-12000.0)/300.0,0.0,1.0)-clamp((time-13000.0)/300.0,0.0,1.0) + clamp((time-22800.0)/200.0,0.0,1.0)-clamp((time-23400.0)/200.0,0.0,1.0));
	float night = clamp((time-12000.0)/300.0,0.0,1.0)-clamp((time-22800.0)/200.0,0.0,1.0);

float mat(float n) { return fract(sin(n)*43758.5453); }

float noise(vec2 x){
    vec2 p = floor(x), f = fract(x); float n = p.x + p.y*32.0;
    return mix(mix(mat(n-1), mat(n),f.x), mix(mat(n+31), mat(n+32),f.x),f.y);
}

float getCNoise(vec4 fragpos, float x, float y, float z) {

	vec4 wrldpos = gbufferModelViewInverse * fragpos / far * 2048;

	float common = (wrldpos.y + length(wrldpos.y))/100, t = frameTimeCounter;  
	float f = 0.25, frq = 1.0, ap = 0.5;

	vec2 wind = vec2(t/5.0)/(x+2.0), camPos = (wrldpos.xz*z*0.5/wrldpos.y);
	vec2 pos = -1+16*(camPos + wind/y); pos /= x/y;

 	f += noise(pos*frq)*ap; frq *= 2.1; ap *= 0.5;
 	f += noise(pos*frq)*ap; frq *= 2.1; ap *= 0.5;
 	f += noise(pos*frq)*ap; frq *= 2.1; ap *= 0.5;
 	f += noise(pos*frq)*ap; frq *= 2.1; ap *= 0.5;

	f += sin(pos.x + t)*0.025; f += sin(pos.y + t)*0.025;

	f /= 0.72; float tex = f, b = 0.875, c = tex - (b+0.2);
	if(c <= 0) c = 0; tex = ((1.0 - pow(b, c))*common);

	float getCNoise = tex * (1-rainStrength);

    return getCNoise;
}

void main() {

	int iswater = int(matflag > 0.04 && matflag < 0.07);
	int land = int(matflag > 0.04);

	vec2 fake_refract = vec2(sin(worldTime/15.0 + texcoord.x*100.0 + texcoord.y*50.0),cos(worldTime/15.0 + texcoord.y*100.0 + texcoord.x*50.0)) * isEyeInWater;

	vec3 color = texture2D(composite, texcoord.st + fake_refract * 0.005).rgb*16.0;

	fragpos = nvec3(gbufferProjectionInverse * vec4(fragpos * 2.0 - 1.0, 1.0));

	vec3 reflectedVector = reflect(normalize(fragpos), normalize(normal));
	reflectedVector = fragpos + reflectedVector * (far-fragpos.z);

	float normalDotEye = dot(normal, normalize(fragpos));
	float fresnel = clamp(pow(1.0 + normalDotEye, 8.0),0.0,1.0)*0.95 + (1-0.95);
	vec4 reflection = vec4(0.0), fpos = vec4(reflectedVector, 1.0);
	vec4 wrldpos = vec4(0.0);
	
	wrldpos = gbufferModelViewInverse * fpos;	
	wrldpos = wrldpos / far * 128.0;

        float horizon = abs(wrldpos.y - texcoord.y);

	float skyPos = max(pow(max(1.0 - horizon/3500.0, 0.01), 8.0)-0.1, 0.0);
	      skyPos = clamp(skyPos, 0.35, 1.0);

	float vista = max(pow(max(1.0 - horizon/500.0, 0.01), 8.0)-0.1, 0.0);

	vec3 skyGradient = ((ambient_color*16 + sunlight*8)/16.0);

	vec3 skyC = vec3(0.15, 0.4, 0.8) * (1.4 + 1.0*night) * (1.0-rainStrength);
	     skyC += vec3(1.6) * rainStrength * (1.0-night);
	     skyC += vec3(0.1) * rainStrength * night;

	vec3 sunC = sunlight * (1.0-night);
	     sunC += vec3(0.15, 0.6, 1.0) * 10.0 * night;

	skyC += (sunlight * 2.0 * getCNoise(fpos, 8., 1.1, 2));

	skyC = skyGradient * skyPos * skyC;
	skyC = mix(skyC, skyGradient * 0.3, vista);

	if (iswater > 0.9 && isEyeInWater == 0) {

	reflection.rgb = mix(skyC*3.0, reflection.rgb, reflection.a/16.0);
	reflection.a = min(reflection.a/16.0 + sky_lightmap,1.0);

	color.rgb += reflection.rgb*fresnel*reflection.a;
	color.rgb += (sunre/16.0)*sunlight*(1-rainStrength)*8.0;
   	}

	float fog = 0.0; 
	fog = 1.0-clamp(exp(-pow(ld(texture2D(depthtex0, texcoord.st).r),3.0)),0.0,1.0);
	
	float time = float(worldTime);
	float transition_fading = 1.0-(clamp((time-12000.0)/300.0,0.0,1.0)-clamp((time-13000.0)/300.0,0.0,1.0) + clamp((time-22800.0)/200.0,0.0,1.0)-clamp((time-23400.0)/200.0,0.0,1.0));
	float night = clamp((time-12000.0)/300.0,0.0,1.0)-clamp((time-22800.0)/200.0,0.0,1.0);

	float fogl = exp(-pow(length(fragpos)/160.0,4.0-(3.0*rainStrength))*8.0);
	float fogfactor =  clamp(fogl,0.0,1.0);
	float rain = rainStrength;

	vec3 fogclr = mix(color.rgb, ambient_color*(1.0-night*rain), 0.2);
	     fogclr += vec3(0.01, 0.02, 0.05)*rain*night; 

	//color.rgb = mix(fogclr, color.rgb, fogfactor);	

	color.rgb += texture2D(gaux4,texcoord.xy).rgb*sqrt(texture2D(gaux4,texcoord.xy).a);

	vec3 curr = Uncharted2Tonemap(color);

	vec3 whiteScale = 1.0f/Uncharted2Tonemap(vec3(W));
	color = curr*whiteScale;

	float saturation = 1.05, avg = (color.r + color.g + color.b);

        color = (((color - avg )*saturation)+avg) ;
	color /= saturation;	
	color = clamp(pow(color,vec3(1.1/2.2)),0.0,1.0);

	gl_FragColor = vec4(color,1.0);
	
}